﻿using System.Windows.Forms;

namespace HeuristicLab.BenchmarkSuite.Views {
  using HeuristicLab.Core.Views;
  using HeuristicLab.MainForm;

  [View("Push Expression Selection Editor")]
  [Content(typeof(IBenchmarkSuiteDataDescriptor), true)]
  public partial class DataEditorView : NamedItemView {
    public DataEditorView() {
      InitializeComponent();

      dataGridView.CellEndEdit += DataGridViewCellEndEdit;
    }

    private void DataGridViewCellEndEdit(object sender, DataGridViewCellEventArgs e) {
      if (Content == null)
        return;

      var row = dataGridView.Rows[e.RowIndex];

      var inputArgs = new string[Content.InputArgumentCount];
      for (var i = 0; i < Content.InputArgumentCount; i++) {
        inputArgs[i] = row.Cells[i].Value.ToString();
      }

      var outputArgs = new string[Content.OutputArgumentCount];
      for (var i = 0; i < Content.OutputArgumentCount; i++) {
        outputArgs[i] = row.Cells[Content.InputArgumentCount + i].Value.ToString();
      }

      var example = Content.Examples[e.RowIndex];

      try {
        var newExample = Content.ParseExample(inputArgs, outputArgs);
        newExample.CopyTo(example);
      }
      catch {
        MessageBox.Show("Unable to parse example!");

        // reset entry
        row.Cells[e.ColumnIndex].Value = e.ColumnIndex < Content.InputArgumentCount
          ? example.InputArgs[e.ColumnIndex]
          : example.OutputArgs[e.ColumnIndex - Content.InputArgumentCount];
      }
    }

    public new IBenchmarkSuiteDataDescriptor Content
    {
      get { return (IBenchmarkSuiteDataDescriptor)base.Content; }
      set
      {
        base.Content = value;
      }
    }

    protected override void OnReadOnlyChanged() {
      base.OnReadOnlyChanged();

      dataGridView.Enabled = !ReadOnly;
    }

    protected override void OnContentChanged() {
      dataGridView.Columns.Clear();
      dataGridView.Rows.Clear();

      if (Content == null) {
        return;
      }

      var cellTemplate = new DataGridViewTextBoxCell();

      for (var i = 1; i <= Content.InputArgumentCount; i++) {
        var column = new DataGridViewColumn {
          HeaderText = string.Format("Input {0}", i),
          AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill,
          CellTemplate = cellTemplate
        };

        dataGridView.Columns.Add(column);
      }

      for (var i = 1; i <= Content.OutputArgumentCount; i++) {
        var column = new DataGridViewColumn {
          HeaderText = string.Format("Output {0}", i),
          AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill,
          CellTemplate = cellTemplate
        };

        dataGridView.Columns.Add(column);
      }

      foreach (var example in Content.Examples) {
        var row = new DataGridViewRow();
        row.HeaderCell.Value = row.Index + 1;

        row.CreateCells(dataGridView);

        for (var i = 0; i < Content.InputArgumentCount; i++) {
          row.Cells[i].Value = example.InputArgs[i];
        }

        for (var i = 0; i < Content.OutputArgumentCount; i++) {
          row.Cells[Content.InputArgumentCount + i].Value = example.OutputArgs[i];
        }

        dataGridView.Rows.Add(row);
      }
    }
  }
}
