﻿using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Data.Pool {
  using System;
  using System.Collections.Concurrent;
  using System.Linq;

  public class ManagedPool<T> : IDisposable where T : class {

    private static readonly ConcurrentStack<LinkedList<T>> Pool = new ConcurrentStack<LinkedList<T>>();
    private readonly Func<T> Factory;
    public readonly int PartitionSize;

    private readonly IList<LinkedList<T>> partitions = new List<LinkedList<T>>();
    private LinkedList<T> currentPartition;
    private LinkedListNode<T> currentNode;

    public ManagedPool(int partitionSize, Func<T> factory) {
      PartitionSize = partitionSize;
      Factory = factory;
    }

    public T Get() {
      if (currentPartition == null || currentPartition.Count == PartitionSize) {
        currentPartition = GetPartition();
        currentNode = currentPartition.First;

        partitions.Add(currentPartition);
      }

      T entry = null;
      if (currentNode == null) {
        entry = Factory();
        currentPartition.AddLast(entry);
      } else {
        entry = currentNode.Value;
        currentNode = currentNode.Next;
      }

      return entry;
    }

    private LinkedList<T> GetPartition() {
      LinkedList<T> partition;

      return Pool.TryPop(out partition) ? partition : new LinkedList<T>();
    }

    public void Dispose() {
      Pool.PushRange(partitions.ToArray());
    }
  }
}
