﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Data.Pool;

  public abstract class StatefulExpression<T> : Expression
  {
    public readonly T State;

    private int hashCode;
    private int HashCode
    {
      get
      {
        if (hashCode == default(int)) hashCode = CalcHashCode();
        return hashCode;
      }
    }

    protected StatefulExpression(T state) {
      this.State = state;
    }

    protected virtual int CalcHashCode() {
      return this.State.GetHashCode();
    }

    public override bool Equals(object obj) {
      if (ReferenceEquals(this, obj))
        return true;

      if (this.GetType() != obj.GetType())
        return false;

      var other = (StatefulExpression<T>)obj;

      return ReferenceEquals(this.State, other.State) ||
             this.GetHashCode() == other.GetHashCode();
    }

    public override int GetHashCode() {
      return HashCode;
    }
  }
}