﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  ///     Removes an indexed item from "deep" in the stack and pushes it on top of the stack.
  ///     The index is taken from the INTEGER stack, and the indexing is done after the index is removed.
  /// </summary>
  /// <typeparam name="T">Stacktype</typeparam>
  public abstract class YankExpression<T> : StatelessExpression {
    public bool Eval(IStack<T> stack, IStack<long> integerStack, int count = 1) {
      if (integerStack.Count == 0) return false;

      var index = integerStack.Top;

      if ((index > stack.Count - count) || (index < 0) || (stack.Count < 2)) return false;

      integerStack.Pop();
      stack.Yank((int)index);

      return true;
    }
  }

  [PushExpression(StackType.Integer, "INTEGER.YANK")]
  public class IntegerYankExpression : YankExpression<long> {
    public override bool Eval(IPushInterpreter interpreter) {
      return this.Eval(interpreter.IntegerStack, interpreter.IntegerStack, 2);
    }
  }

  [PushExpression(StackType.Float, "FLOAT.YANK")]
  public class FloatYankExpression : YankExpression<double> {
    public override bool Eval(IPushInterpreter interpreter) {
      return this.Eval(interpreter.FloatStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackType.Boolean, "BOOLEAN.YANK")]
  public class BooleanYankExpression : YankExpression<bool> {
    public override bool Eval(IPushInterpreter interpreter) {
      return this.Eval(interpreter.BooleanStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackType.Name, "NAME.YANK")]
  public class NameYankExpression : YankExpression<string> {
    public override bool Eval(IPushInterpreter interpreter) {
      return this.Eval(interpreter.NameStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackType.Exec, "EXEC.YANK")]
  public class ExecYankExpression : YankExpression<Expression> {
    public override bool Eval(IPushInterpreter interpreter) {
      return this.Eval(interpreter.ExecStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackType.Code, "CODE.YANK")]
  public class CodeYankExpression : YankExpression<Expression> {
    public override bool Eval(IPushInterpreter interpreter) {
      return this.Eval(interpreter.CodeStack, interpreter.IntegerStack);
    }
  }
}