﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter {
  using System.Collections.Generic;
  using System.Threading;
  using System.Threading.Tasks;
  using Core;
  using Expressions;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Configuration;

  using Stack;

  public interface IPushInterpreter {
    IRandom Random { get; set; }

    IStack<Expression> CodeStack { get; }
    IStack<Expression> ExecStack { get; }
    IStack<string> NameStack { get; }
    IStack<bool> BooleanStack { get; }
    IStack<long> IntegerStack { get; }
    IStack<double> FloatStack { get; }

    IDictionary<string, Expression> CustomExpressions { get; }

    IReadonlyPushConfiguration Configuration { get; }

    bool IsNameQuoteFlagSet { get; set; }

    void Clear();
    void Run(string code, bool stepwise = false);
    void Run(Expression program, bool stepwise = false);
    void Run(PushProgram program, bool stepwise = false);
    Task RunAsync(PushProgram program, bool paused = false, CancellationToken token = default(CancellationToken));
    Task RunAsync(Expression program, bool paused = false, CancellationToken token = default(CancellationToken));
    Task RunAsync(string code, bool paused = false, CancellationToken token = default(CancellationToken));
    Task AbortAndResetAsync();
    Task AbortAsync();
    Task PauseAsync();
    void Resume();
    Task ResumeAsync();

    bool Step();
    void PrintStacks();
  }
}