﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter {
  using System;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Data.Pool;
  using HeuristicLab.Core;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Configuration;
  using HeuristicLab.Random;

  public class PushInterpreterPool {
    private readonly ObjectPool<PooledPushInterpreter> pool;

    public PushInterpreterPool(PushConfiguration config = null)
      : this(Environment.ProcessorCount * 2, config) {
    }

    public PushInterpreterPool(int size, PushConfiguration config = null) {
      this.PushGpConfiguration = config ?? new PushConfiguration();

      this.pool = new ObjectPool<PooledPushInterpreter>(() => new PooledPushInterpreter(this, this.PushGpConfiguration), size);
    }

    public PushConfiguration PushGpConfiguration { get; private set; }

    public PooledPushInterpreter GetInstance(IRandom random = null) {
      var interpreter = this.pool.Allocate();
      interpreter.Random = random ?? new FastRandom();

      return interpreter;
    }

    public void Free(PooledPushInterpreter interpreter) {
      this.pool.Free(interpreter);
    }
  }
}
