﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Problem {
  using HeuristicLab.BenchmarkSuite;
  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Encodings.IntegerVectorEncoding;
  using HeuristicLab.Optimization;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Configuration;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Expressions;

  [StorableClass]
  public class PushSolution : NamedItem {
    [Storable]
    public readonly double Quality;
    [Storable]
    public readonly IntegerVector IntegerVector;
    [Storable]
    public readonly IBenchmarkSuiteDataDescriptor DataDescriptor;
    [Storable]
    public readonly IRandom Random;
    [Storable]
    public readonly IReadonlyPushConfiguration Config;

    public PushSolution(IntegerVector integerVector, double quality, IBenchmarkSuiteDataDescriptor dataDescriptor, IRandom random, IReadonlyPushConfiguration config)
      : base("Solution", "A push solution.") {
      IntegerVector = integerVector;
      Quality = quality;
      DataDescriptor = dataDescriptor;
      Random = random;
      Config = config;
    }

    public PushSolution(PushSolution origin, Cloner cloner) : base(origin, cloner) {
      IntegerVector = origin.IntegerVector;
      Quality = origin.Quality;
    }

    [StorableConstructor]
    private PushSolution(bool deserializing) : base(deserializing) { }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PushSolution(this, cloner);
    }
  }
}
