﻿using System.Collections.Generic;

namespace TestPooling.Pool {
  using System;
  using System.Collections.Concurrent;
  using System.IO;
  using System.Linq;
  using System.Runtime.Serialization.Formatters.Binary;

  public class ManagedPool2<T> : IDisposable where T : class {

    private static readonly ConcurrentStack<T[]> Pool = new ConcurrentStack<T[]>();

    private readonly T[] DummyPartition;
    private readonly Func<T> Factory;
    public readonly int PartitionSize;

    private readonly IList<T[]> partitions = new List<T[]>();
    private T[] currentPartition;
    private int entryIndex;

    public ManagedPool2(int partitionSize, T[] dummyPartition, Func<T> factory) {
      PartitionSize = partitionSize;
      Factory = factory;
      entryIndex = partitionSize;

      DummyPartition = dummyPartition;
    }

    public T Get() {
      if (entryIndex == PartitionSize) {
        currentPartition = GetPartition();
        partitions.Add(currentPartition);
        entryIndex = 0;
      }

      return currentPartition[entryIndex++];
    }

    private T[] GetPartition() {
      //T[] partition;

      //partition = Pool.TryPop(out partition) ? partition : new T[PartitionSize];

      //for (var i = 0; i < PartitionSize; i++) {
      //  partition[i] = Factory();
      //}

      //return partition;

      T[] partition;
      if (Pool.TryPop(out partition)) return partition;

      //partition = new T[PartitionSize];
      //DummyPartition.CopyTo(partition, 0);

      return DeepArrayCopy(DummyPartition);
    }

    private static T[] DeepArrayCopy(object objectToCopy) {
      using (var memoryStream = new MemoryStream()) {
        var binaryFormatter = new BinaryFormatter();

        binaryFormatter.Serialize(memoryStream, objectToCopy);
        memoryStream.Seek(0, SeekOrigin.Begin);

        return (T[])binaryFormatter.Deserialize(memoryStream);
      }
    }

    public void Dispose() {
      Pool.PushRange(partitions.ToArray());
    }
  }
}
