﻿using System.Threading.Tasks;

namespace TestPooling {
  using System;
  using System.Diagnostics;

  using TestPooling.Pool;

  class Program {
    private const int Count = 3000;
    static void Main(string[] args) {
      Test1();
      Test1_2();
      Test3();
    }

    private static void Test1() {
      var sw = new Stopwatch();

      sw.Start();
      //Parallel.For(0, 3, k => {
      Parallel.For(0, 10000, i => {
        using (var pool = new ManagedPool<Person>(Count, () => new Person())) {
          for (var j = 0; j < 3000; j++) {
            var person = pool.Get();
            person.Age = i + j;
            person.Name = string.Format("{0} - {1}", i, j);
          }
        }
      });
      //});
      sw.Stop();

      Console.WriteLine("1 - Duration: {0} - {1}", sw.ElapsedTicks, sw.Elapsed.TotalSeconds);
    }

    private static void Test1_2() {
      var sw = new Stopwatch();
      sw.Start();

      var dummyPartition = new Person[Count];

      for (var i = 0; i < Count; i++) {
        dummyPartition[i] = new Person();
      }

      //Parallel.For(0, 3, k => {
      Parallel.For(0, 10000, i => {
        using (var pool = new ManagedPool2<Person>(Count, dummyPartition, () => new Person())) {
          for (var j = 0; j < 3000; j++) {
            var person = pool.Get();
            person.Age = i + j;
            person.Name = string.Format("{0} - {1}", i, j);
          }
        }
      });
      //});
      sw.Stop();

      //var p1 = new ManagedPool2<Person>(Count, dummyPartition, () => new TestPooling.Person());
      //var p2 = new ManagedPool2<Person>(Count, dummyPartition, () => new TestPooling.Person());
      //var pa1 = p1.Get();
      //var pa2 = p2.Get();

      Console.WriteLine("1_2 - Duration: {0} - {1}", sw.ElapsedTicks, sw.Elapsed.TotalSeconds);
    }

    private static void Test4() {
      var sw = new Stopwatch();

      sw.Start(); ;
      var pool = new ManagedPool<Person>(512, () => new Person());

      Parallel.For(0, 10000, i => {
        for (var j = 0; j < 3000; j++) {
          var person = pool.Get();
          person.Age = i + j;
          person.Name = string.Format("{0} - {1}", i, j);
        }
        pool.Dispose();
      });
      sw.Stop();

      Console.WriteLine("1 - Duration: {0}", sw.ElapsedTicks);
    }

    private static void Test3() {
      var count = 0;
      var sw = new Stopwatch();

      sw.Start();
      Parallel.For(0, 10000, i => {
        var list = new Person[3000];

        for (var j = 0; j < 3000; j++) {
          list[j] = new Person { Age = i + j, Name = string.Format("{0} - {1}", i, j) };
        }

        count += list.Length;
      });
      sw.Stop();

      Console.WriteLine("3 - Duration: {0} - {1}", sw.ElapsedTicks, sw.Elapsed.TotalSeconds);
      Console.WriteLine("Count: {0}", count);
    }

    private static void Test2() {
      var pool = new ObjectPool<Person>(() => new Person(), 2048);

      Parallel.For(0, 10000, i => {
        for (var j = 0; j < 3000; j++) {
          var person = pool.Allocate();
          person.Age = i + j;
          person.Name = string.Format("{0} - {1}", i, j);
        }

        //pool.Free()
      });
    }
  }

  [Serializable]
  class Person {
    public string Name { get; set; }
    public int Age { get; set; }
  }
}
